package info.textgrid.middleware.test.online.tg;

import java.io.IOException;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import org.apache.cxf.jaxrs.client.Client;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.test.online.OaipmhUtilitiesOnline;

/**
 * <p>
 * Some online tests for the TextGrid OAIMPH service --> verb=ListRecords <--
 * </p>
 * 
 * <p>
 * Please set PROPERTIES_FILE and TEST_ALL_PAGES in class <b>OAIPMHUtilitiesOnline</b>!
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-28
 * @since 2022-09-12
 */
// @Ignore
public class TestTGListRecordsOnline {

  // **
  // STATICS
  // **

  // Some JAXRS things.
  private static String oaipmhEndpoint;
  static Client oaipmhWebClient;

  // Properties
  private static String checkListRecordsDCSet;
  private static String checkListRecordsDCFrom;
  private static String checkListRecordsDCUntil;
  private static String checkListRecordsDATACITEFrom;
  private static String checkListRecordsDATACITEUntil;
  private static String checkListRecordsIDIOMFrom;
  private static String checkListRecordsIDIOMUntil;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Get properties.
    Properties p =
        OaipmhUtilitiesOnline.getPropertiesFromFile(OaipmhUtilitiesOnline.PROPERTIES_FILE);

    // Set properties.
    oaipmhEndpoint = p.getProperty("oaipmhEndpoint");

    checkListRecordsDCSet = p.getProperty("checkListRecordsDCSet");
    checkListRecordsDCFrom = p.getProperty("checkListRecordsDCFrom");
    checkListRecordsDCUntil = p.getProperty("checkListRecordsDCUntil");

    checkListRecordsDATACITEFrom = p.getProperty("checkListRecordsDATACITEFrom");
    checkListRecordsDATACITEUntil = p.getProperty("checkListRecordsDATACITEUntil");

    checkListRecordsIDIOMFrom = p.getProperty("checkListRecordsIDIOMFrom");
    checkListRecordsIDIOMUntil = p.getProperty("checkListRecordsIDIOMUntil");

    // Get web client from endpoint.
    oaipmhWebClient = OaipmhUtilitiesOnline.getOAIPMHWEebClient(oaipmhEndpoint);
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {
    //
  }

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCMorePages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsOAIDCMorePages()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_DC_PREFIX,
        18, 100,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCSetMorePages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsOAIDCSetMorePages()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        checkListRecordsDCSet,
        OaipmhUtilitiesOnline.OAI_DC_PREFIX,
        20, 100,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCSetMorePagesNoProjectPrefix() throws IOException {

    System.out.println(
        OaipmhUtilitiesOnline.TESTING + "testListRecordsOAIDCSetMorePagesNoProjectPrefix()");

    String setWithoutProjectPRefix = checkListRecordsDCSet.replace("project:", "");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        setWithoutProjectPRefix,
        OaipmhUtilitiesOnline.OAI_DC_PREFIX,
        1, 0,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCAllPages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsOAIDCAllPages()");

    if (OaipmhUtilitiesOnline.TEST_ALL_PAGES) {

      OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
          OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
          OaipmhUtilitiesOnline.NO_SET,
          OaipmhUtilitiesOnline.OAI_DC_PREFIX,
          0, 100,
          OaipmhUtilitiesOnline.NO_THREAD_NAME,
          OaipmhUtilitiesOnline.NO_FROM,
          OaipmhUtilitiesOnline.NO_UNTIL,
          OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
          OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

      System.out.println(OaipmhUtilitiesOnline.OK);
    } else {
      System.out.println("...skipping");
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCMorePagesFromUntil() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsOAIDCMorePagesFromUntil()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        checkListRecordsDCSet,
        OaipmhUtilitiesOnline.OAI_DC_PREFIX,
        0, 100,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsDCFrom,
        checkListRecordsDCUntil,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test(expected = IOException.class)
  public void testListRecordsIDIOMMETSSomePagesAndMetadataFormatWithRestok() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING
        + "testListRecordsIDIOMMETSSomePagesAndMetadataFormatWithRestok()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        3, 30,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.METADATA_FORMAT_WITH_RESTOK, OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSAllPages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSAllPages()");

    if (OaipmhUtilitiesOnline.TEST_ALL_PAGES) {

      OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
          OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
          OaipmhUtilitiesOnline.NO_SET,
          OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
          0, 30,
          OaipmhUtilitiesOnline.NO_THREAD_NAME,
          OaipmhUtilitiesOnline.NO_FROM,
          OaipmhUtilitiesOnline.NO_UNTIL,
          OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
          OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

      System.out.println(OaipmhUtilitiesOnline.OK);
    } else {
      System.out.println("...skipping");
    }
  }

  /**
   * NOTE The amount of records will change due to changes on the data! Please check from and until
   * values before!
   *
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSMorePagesFromUntil() throws IOException {

    System.out
        .println(OaipmhUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSMorePagesFromUntil()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        6, 30,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsIDIOMFrom,
        checkListRecordsIDIOMUntil,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSMorePages() throws IOException {

    System.out
        .println(OaipmhUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSMorePages()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        6, 30,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEMorePages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsDATACITEMorePages()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_DATACITE_PREFIX,
        15, 100,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        OaipmhUtilitiesOnline.NO_FROM,
        OaipmhUtilitiesOnline.NO_UNTIL,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEAllPages() throws IOException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testListRecordsDATACITEAllPages()");

    if (OaipmhUtilitiesOnline.TEST_ALL_PAGES) {
      OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
          OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
          OaipmhUtilitiesOnline.NO_SET,
          OaipmhUtilitiesOnline.OAI_DATACITE_PREFIX,
          0, 100,
          OaipmhUtilitiesOnline.NO_THREAD_NAME,
          OaipmhUtilitiesOnline.NO_FROM,
          OaipmhUtilitiesOnline.NO_UNTIL,
          OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
          OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

      System.out.println(OaipmhUtilitiesOnline.OK);
    } else {
      System.out.println("...skipping");
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEMorePagesFromUntil() throws IOException {

    System.out
        .println(OaipmhUtilitiesOnline.TESTING + "testListRecordsDATACITEMorePagesFromUntil()");

    OaipmhUtilitiesOnline.examineTGList(oaipmhWebClient,
        OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
        OaipmhUtilitiesOnline.NO_SET,
        OaipmhUtilitiesOnline.OAI_DATACITE_PREFIX,
        15, 100,
        OaipmhUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsDATACITEFrom,
        checkListRecordsDATACITEUntil,
        OaipmhUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK,
        OaipmhUtilitiesOnline.NO_ERROR_EXPECTED);

    System.out.println(OaipmhUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   * @throws ExecutionException
   * @throws InterruptedException
   */
  @Test
  public void testListRecordsConcurrentlyIDIOMMETSMorePages()
      throws IOException, InterruptedException, ExecutionException {

    System.out
        .println(OaipmhUtilitiesOnline.TESTING + "testListRecordsConcurrentlyIDIOMMETSMorePages()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
            2, 30,
            "IDIOM1"));
    Future<Boolean> f2 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
            3, 30,
            "IDIOM2"));
    Future<Boolean> f3 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
            4, 30,
            "IDIOM3"));

    executor.shutdown();

    System.out.println(OaipmhUtilitiesOnline.OK + ": [IDIOM1]=" + f1.get() + ", [IDIOM2]="
        + f2.get() + ", [IDIOM3]=" + f3.get());
  }


  /**
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testRestokConcurrentlyListRecordsDC()
      throws InterruptedException, ExecutionException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testRestokConcurrentlyListRecordsDC()");

    ExecutorService executor = Executors.newFixedThreadPool(4);

    Future<Boolean> f1 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            44, 100,
            "DC1"));
    Future<Boolean> f2 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            33, 100,
            "DC2"));
    Future<Boolean> f3 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            66, 100,
            "DC3"));
    Future<Boolean> f4 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            28, 100,
            "DC4"));

    executor.shutdown();

    System.out.println(
        OaipmhUtilitiesOnline.OK + ": [DC1]=" + f1.get() + ", [DC2]=" + f2.get() + ", [DC3]="
            + f3.get() + ", [DC4]=" + f4.get());
  }

  /**
   * @throws IOException
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testRestokConcurrentlyListRecords() throws InterruptedException, ExecutionException {

    System.out.println(OaipmhUtilitiesOnline.TESTING + "testRestokConcurrentlyListRecords()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            6, 100,
            "B1"));
    Future<Boolean> f2 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            7, 100,
            "B2"));
    Future<Boolean> f3 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            8, 100,
            "B3"));
    Future<Boolean> f4 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            9, 100,
            "B4"));

    executor.shutdown();

    System.out.println(OaipmhUtilitiesOnline.OK + ": [B1]=" + f1.get() + ", [B2]=" + f2.get()
        + ", [B3]=" + f3.get() + ", [B4]=" + f4.get());
  }

  /**
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testListRecordsRestokConcurrentlyDCAndIDIOMMETS()
      throws InterruptedException, ExecutionException {

    System.out.println(
        OaipmhUtilitiesOnline.TESTING + "testListRecordsRestokConcurrentlyDCAndIDIOMMets()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            34, 100,
            "DC1"));
    Future<Boolean> f2 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_DC_PREFIX,
            27, 100,
            "DC2"));
    Future<Boolean> f3 = executor.submit(
        new TGOaipmhResumptionTokenThread(oaipmhWebClient,
            OaipmhUtilitiesOnline.VERB_LIST_RECORDS,
            OaipmhUtilitiesOnline.NO_SET,
            OaipmhUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
            5, 30,
            "IDIOM"));

    executor.shutdown();

    System.out.println(OaipmhUtilitiesOnline.OK + ": [DC1]=" + f1.get() + ", [DC2]=" + f2.get()
        + ", [IOIOM]=" + f3.get());
  }

}
